#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Anki Add-on Builder
#
# Copyright (C)  2016-2019 Aristotelis P. <https://glutanimate.com/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version, with the additions
# listed at the end of the license file that accompanied this program.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# NOTE: This program is subject to certain additional terms pursuant to
# Section 7 of the GNU Affero General Public License.  You should have
# received a copy of these additional terms immediately following the
# terms and conditions of the GNU Affero General Public License that
# accompanied this program.
#
# If not, please request a copy through one of the means of contact
# listed here: <https://glutanimate.com/contact/>.
#
# Any modifications to this file must keep this entire header intact.


from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import sys
import logging
import argparse

from . import PATH_ROOT, COPYRIGHT_MSG, DIST_TYPES
from .config import Config, PATH_CONFIG
from .builder import AddonBuilder
from .ui import UIBuilder
from .utils import PY2K


# Checks
##############################################################################

def validate_cwd():
    required = (PATH_ROOT / "src", PATH_CONFIG)
    for path in required:
        if not path.exists():
            print("Error: {dir} not found. Please run this script from "
                  "the project root.".format(dir=path.name))
            return False
    return True


# Entry points
##############################################################################

def build(args):
    targets = [args.target] if args.target != "all" else Config()["targets"]
    dists = [args.dist] if args.dist != "all" else DIST_TYPES

    builder = AddonBuilder(version=args.version)
    
    cnt = 1
    total = len(targets) * len(dists)
    for target in targets:
        for dist in dists:
            logging.info("\n=== Build task %s/%s ===", cnt, total)
            builder.build(target=target, disttype=dist)
            cnt += 1


def ui(args):
    targets = [args.target] if args.target != "all" else Config()["targets"]

    builder = UIBuilder(root=PATH_ROOT)

    cnt = 1
    total = len(targets)
    for target in targets:
        logging.info("\n=== Build task %s/%s ===\n", cnt, total)
        builder.build(target=target)
        cnt += 1


def clean(args):
    builder = AddonBuilder()
    return builder.clean()


# Argument parsing
##############################################################################

def construct_parser():
    parser = argparse.ArgumentParser()
    if not PY2K:
        parser.set_defaults(func=lambda x: parser.print_usage())
    subparsers = parser.add_subparsers()

    parser.add_argument("-v", "--verbose",
                        help="Enable verbose output",
                        required=False, action="store_true")

    target_parent = argparse.ArgumentParser(add_help=False)
    target_parent.add_argument("-t", "--target",
                               help="Anki version to build for",
                               type=str, default="anki21",
                               choices=["anki21", "anki20", "all"])

    dist_parent = argparse.ArgumentParser(add_help=False)
    dist_parent.add_argument("-d", "--dist",
                             help="Distribution channel to build for",
                             type=str, default="local",
                             choices=["local", "ankiweb", "all"])

    build_group = subparsers.add_parser(
        "build", parents=[target_parent, dist_parent],
        help="Build and package add-on for distribution"
    )
    build_group.add_argument(
        "version", nargs="?", help="Version to build as a git reference "
        "(e.g. 'v1.2.0' or 'd338f6405'). "
        "Special keywords: 'current' – latest commit, 'release' – latest tag. "
        "Leave empty to build latest tag.")
    build_group.set_defaults(func=build)

    ui_group = subparsers.add_parser(
        "ui", parents=[target_parent],
        help="Compile add-on user interface files"
    )
    ui_group.set_defaults(func=ui)

    clean_group = subparsers.add_parser(
        "clean", help="Clean leftover build files")
    clean_group.set_defaults(func=clean)

    return parser


# Main
##############################################################################

def main():
    print(COPYRIGHT_MSG)

    # Checks
    if not validate_cwd():
        sys.exit(1)

    # Argument parsing

    parser = construct_parser()
    args = parser.parse_args()

    # Logging

    if args.verbose:
        level = logging.DEBUG
    else:
        level = logging.INFO

    logging.basicConfig(stream=sys.stdout, level=level, format="%(message)s")

    # Run

    try:
        args.func(args)
    except AttributeError as e:
        print(e)
        parser.print_help()
        parser.exit()

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Anki Add-on Builder
#
# Copyright (C)  2016-2019 Aristotelis P. <https://glutanimate.com/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version, with the additions
# listed at the end of the license file that accompanied this program.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# NOTE: This program is subject to certain additional terms pursuant to
# Section 7 of the GNU Affero General Public License.  You should have
# received a copy of these additional terms immediately following the
# terms and conditions of the GNU Affero General Public License that
# accompanied this program.
#
# If not, please request a copy through one of the means of contact
# listed here: <https://glutanimate.com/contact/>.
#
# Any modifications to this file must keep this entire header intact.

"""
Basic Git interface
"""

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import logging

from .utils import call_shell

class Git(object):

    def version(self, commit=False):
        logging.info("Getting Git version info...")
        cmd = "git describe HEAD --tags"
        if commit is False:
            cmd += " --abbrev=0"
        return call_shell(cmd)

    def archive(self, version, outdir):
        logging.info("Exporting Git archive...")
        if not outdir or not version:
            return False
        cmd = "git archive --format tar {vers} | tar -x -C {outdir}/".format(
            vers=version, outdir=outdir)
        return call_shell(cmd)

    def modtime(self, version):
        return int(call_shell("git log -1 -s --format=%ct {}".format(version)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Anki Add-on Builder
#
# Copyright (C)  2016-2019 Aristotelis P. <https://glutanimate.com/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version, with the additions
# listed at the end of the license file that accompanied this program.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# NOTE: This program is subject to certain additional terms pursuant to
# Section 7 of the GNU Affero General Public License.  You should have
# received a copy of these additional terms immediately following the
# terms and conditions of the GNU Affero General Public License that
# accompanied this program.
#
# If not, please request a copy through one of the means of contact
# listed here: <https://glutanimate.com/contact/>.
#
# Any modifications to this file must keep this entire header intact.

"""
Project config parser
"""

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import json
import logging

import jsonschema
from jsonschema.exceptions import ValidationError

from . import PATH_ROOT, PATH_PACKAGE
from .git import Git

PATH_CONFIG = PATH_ROOT / "addon.json"

class Config(object):

    """
    Initialize a read-only add-on config object
    """
    
    with (PATH_PACKAGE / "schema.json").open("r", encoding="utf-8") as f:
        _schema = json.loads(f.read())

    def __init__(self):
        try:
            with PATH_CONFIG.open(encoding="utf-8") as f:
                self._dict = json.loads(f.read())
            jsonschema.validate(self._dict, self._schema)
        except (IOError, OSError, ValueError, ValidationError):
            self._dict = None
            logging.error("Error: Could not read '{}'. Traceback follows "
                          "below:\n".format(PATH_CONFIG.name))
            raise

    def get(self, name):
        return self._dict.get(name)

    def __getitem__(self, name):
        return self._dict[name]

    def __str__(self):
        return self._dict.__str__()

    def __repr__(self):
        return self._dict.__repr__()

    def manifest(self, version, disttype="local"):
        config = self._dict
        manifest = {
            "name": config["display_name"],
            "package": config["module_name"],
            "ankiweb_id": config["ankiweb_id"],
            "author": config["author"],
            "version": version,
            "homepage": config.get("homepage", ""),
            "conflicts": config["conflicts"],
            "mod": Git().modtime(version)
        }

        # Update values for distribution type
        if disttype == "local" and config["ankiweb_id"]:
            manifest["conflicts"].insert(0, config["ankiweb_id"])
        elif disttype == "ankiweb" and config["module_name"]:
            manifest["conflicts"].insert(0, config["module_name"])
            # this is inconsistent, but we can't do much else when
            # ankiweb_id is still unknown (i.e. first upload):
            manifest["package"] = config["ankiweb_id"] or config["module_name"]

        return manifest



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Anki Add-on Builder
#
# Copyright (C)  2016-2019 Aristotelis P. <https://glutanimate.com/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version, with the additions
# listed at the end of the license file that accompanied this program.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# NOTE: This program is subject to certain additional terms pursuant to
# Section 7 of the GNU Affero General Public License.  You should have
# received a copy of these additional terms immediately following the
# terms and conditions of the GNU Affero General Public License that
# accompanied this program.
#
# If not, please request a copy through one of the means of contact
# listed here: <https://glutanimate.com/contact/>.
#
# Any modifications to this file must keep this entire header intact.

"""
Utility functions
"""

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import sys
import subprocess
import logging

from . import PATH_ROOT

# Python 2 is needed to support Anki 2.0 development environments
PY2K = sys.version_info < (3, 0)
unicode = unicode if PY2K else str  # noqa:F821

def call_shell(command, echo=False, **kwargs):
    try:
        out = subprocess.check_output(command, shell=True, **kwargs)
        decoded = out.decode("utf-8").strip()
        if echo:
            logging.info(decoded)
        return decoded
    except subprocess.CalledProcessError as e:
        logging.error("Error while running command: '{command}'".format(
                      command=command))
        logging.error(e.output.decode("utf-8"))
        sys.exit(1)


def purge(path, patterns, recursive=False):
    """Wrapper for GNU find that deletes files matching pattern

    Arguments:
        path {str} -- Path to look through
        patterns {list} -- List of patterns to delete

    Keyword Arguments:
        recursive {bool} -- Whether to search recursively (default: {False})
    """
    if not path or not patterns:
        return False
    pattern_string = " -o ".join("-name '{}'".format(p) for p in patterns)
    pattern_string = "\( {} \)".format(pattern_string)
    depth = "-maxdepth 1" if not recursive else ""
    cmd = 'find "{path}" {depth} {pattern_string} -delete'.format(
        path=path, depth=depth, pattern_string=pattern_string)
    return call_shell(cmd)


def copy_recursively(source, target):
    if not source or not target:
        return False
    return call_shell('cp -r "{source}" "{target}"'.format(
                      source=source, target=target))


def relpath(path):
    return path.relative_to(PATH_ROOT)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='aab',
    version='0.1.2',
    description='Anki Add-on Builder',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/glutanimate/anki-addon-builder',
    author='Aristotelis P. (Glutanimate)',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        "Operating System :: POSIX",
        "Operating System :: MacOS",
    ],
    keywords='anki development build-tools',
    packages=["aab"],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, <4',
    install_requires=["jsonschema", "whichcraft",
                      "pathlib;python_version<'3.4'"],
    # e.g. $ pip install aab[anki21]
    extras_require={
        'anki21': ['PyQt5>=5.12']
    },
    entry_points={
        'console_scripts': [
            'aab=aab.cli:main',
        ],
    },
    package_data={
        'aab': ['schema.json']
    },
    project_urls={
        'Bug Reports': 'https://github.com/glutanimate/anki-addon-builder/issues',
        'Funding': 'https://glutanimate.com/support-my-work/',
        'Source': 'https://github.com/glutanimate/anki-addon-builder',
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Anki Add-on Builder
#
# Copyright (C)  2016-2019 Aristotelis P. <https://glutanimate.com/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version, with the additions
# listed at the end of the license file that accompanied this program.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# NOTE: This program is subject to certain additional terms pursuant to
# Section 7 of the GNU Affero General Public License.  You should have
# received a copy of these additional terms immediately following the
# terms and conditions of the GNU Affero General Public License that
# accompanied this program.
#
# If not, please request a copy through one of the means of contact
# listed here: <https://glutanimate.com/contact/>.
#
# Any modifications to this file must keep this entire header intact.

"""
Main Add-on Builder
"""

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import os
import shutil
import json
import zipfile
import logging

from . import PATH_DIST, PATH_ROOT
from .config import Config
from .git import Git
from .ui import UIBuilder
from .utils import purge, unicode, copy_recursively

class AddonBuilder(object):

    _trash_patterns = ["*.pyc", "*.pyo", "__pycache__"]
    _paths_licenses = [PATH_DIST, PATH_DIST / "resources"]
    _path_optional_icons = PATH_ROOT / "resources" / "icons" / "optional"
    _path_changelog = PATH_DIST / "CHANGELOG.md"

    def __init__(self, version=None, callback_archive=None):
        self._version = self._get_version(version)
        self._callback_archive = callback_archive
        self._config = Config()
        self._path_dist_module = PATH_DIST / \
            "src" / self._config["module_name"]

    def build(self, target="anki21", disttype="local", pyenv=None):
        
        logging.info("\n--- Building %s %s for %s/%s ---\n",
                     self._config["display_name"], self._version,
                     target, disttype)
        
        self.clean()

        PATH_DIST.mkdir(parents=True)
        Git().archive(self._version, PATH_DIST)

        self._copy_licenses()
        if self._path_changelog.exists():
            self._copy_changelog()
        if self._path_optional_icons.exists():
            self._copy_optional_icons()
        if self._callback_archive:
            self._callback_archive()

        self._write_manifest(disttype)
        self._build_ui(target, pyenv)
        
        return self._package(target, disttype)

    def clean(self):
        logging.info("Cleaning repository...")
        if PATH_DIST.exists():
            shutil.rmtree(unicode(PATH_DIST))
        purge(".", self._trash_patterns, recursive=True)

    def _build_ui(self, target, pyenv):
        logging.info("Building UI...")
        UIBuilder(root=PATH_DIST).build(target=target, pyenv=pyenv)

    def _get_version(self, version):
        if version is None or version == "release":
            return Git().version()
        elif version == "current":
            return Git().version(commit=True)
        return version

    def _package(self, target, disttype):
        logging.info("Packaging add-on...")
        config = self._config

        if target == "anki21":
            to_zip = self._path_dist_module
            ext = "ankiaddon"
        else:
            to_zip = PATH_DIST / "src"
            ext = "zip"

        out_name = "{repo_name}-{version}-{target}{dist}.{ext}".format(
            repo_name=config["repo_name"], version=self._version,
            target=target,
            dist="" if disttype == "local" else "-" + disttype,
            ext=ext
        )

        out_path = PATH_ROOT / "build" / out_name

        if out_path.exists():
            out_path.unlink()

        with zipfile.ZipFile(unicode(out_path),
                             "w", zipfile.ZIP_DEFLATED) as myzip:
            rootlen = len(unicode(to_zip)) + 1
            for root, dirs, files in os.walk(unicode(to_zip)):
                for file in files:
                    path = os.path.join(root, file)
                    myzip.write(path, path[rootlen:])

        logging.info("Package saved as {out_name}".format(out_name=out_name))
        logging.info("Done.")
        
        return out_path

    def _write_manifest(self, disttype):
        logging.info("Writing manifest...")
        contents = self._config.manifest(self._version, disttype=disttype)
        path = self._path_dist_module / "manifest.json"
        with path.open("w", encoding="utf-8") as f:
            f.write(unicode(json.dumps(contents, indent=4,
                                       sort_keys=False,
                                       ensure_ascii=False)))

    def _copy_licenses(self):
        logging.info("Copying licenses...")
        for path in self._paths_licenses:
            if not path.is_dir():
                continue
            for file in path.glob("LICENSE*"):
                target = (self._path_dist_module /
                          "{stem}.txt".format(stem=file.stem))
                shutil.copyfile(unicode(file), unicode(target))

    def _copy_changelog(self):
        logging.info("Copying changelog...")
        target = self._path_dist_module / "CHANGELOG.md"
        shutil.copy(unicode(self._path_changelog), unicode(target))

    def _copy_optional_icons(self):
        logging.info("Copying additional icons...")
        copy_recursively(self._path_optional_icons,
                         PATH_DIST / "resources" / "icons" / "")
